/* --------------------------------------------------------------------------------
 #
 #	4DPlugin.c
 #	source generated by 4D Plugin Wizard
 #	Project : 4D OpenSSL
 #	author : miyako
 #	09/10/23
 #
 # --------------------------------------------------------------------------------*/

#include "4DPluginAPI.h"
#include "4DPlugin.h"

#if VERSIONWIN	
#include <windows.h>
#include <stdio.h>
#endif

#include "PA_Frameworks.h"
#include "PA_UTF8.h"
#include "open_ssl_support.h"

#if VERSIONMAC
CFBundleRef openssl;
#else
HMODULE openssl;
#endif

void PluginMain( long selector, PA_PluginParameters params )
{
	switch( selector )
	{
		case kInitPlugin :
			InitPlugin();
			break;
		case kDeinitPlugin :
			DeInitPlugin();
			break;
			
// --- crypto

		case 1 :
			SHA256( params );
			break;

		case 2 :
			MD5( params );
			break;

		case 3 :
			SHA1( params );
			break;

		case 4 :
			SHA384( params );
			break;

		case 5 :
			SHA512( params );
			break;

		case 6 :
			HMACMD5( params );
			break;

		case 7 :
			HMACSHA1( params );
			break;

		case 8 :
			HMACSHA256( params );
			break;

		case 9 :
			HMACSHA384( params );
			break;

		case 10 :
			HMACSHA512( params );
			break;

	}
}
void DeInitPlugin()
{
	if(openssl){
#if VERSIONMAC	
		CFRelease( openssl );
#else
		FreeLibrary( openssl );
#endif
	}
}

void InitPlugin()
{
#if VERSIONMAC	
	openssl = CFBundleCopyFourdPrivateFrameworkByName( CFSTR("OpenSSL.framework") );
#else
	wchar_t thisPath[ _MAX_PATH ] = {0};
	GetModuleFileName(NULL, thisPath, _MAX_PATH);
	openssl = LoadLibraryEx( L"libeay32.dll" , NULL, LOAD_WITH_ALTERED_SEARCH_PATH );
#endif
	
	if(openssl){
		
		EVP_md5 = GET_PROC(EVP_MD_FUNC_MD, openssl, "EVP_md5");
		EVP_sha1 = GET_PROC(EVP_MD_FUNC_MD, openssl, "EVP_sha1");
		EVP_sha256 = GET_PROC(EVP_MD_FUNC_MD, openssl, "EVP_sha256");
		EVP_sha384 = GET_PROC(EVP_MD_FUNC_MD, openssl, "EVP_sha384");
		EVP_sha512 = GET_PROC(EVP_MD_FUNC_MD, openssl, "EVP_sha512");	
		
		EVP_MD_CTX_create = GET_PROC(EVP_MD_FUNC, openssl, "EVP_MD_CTX_create");
		EVP_MD_CTX_init = GET_PROC(EVP_MD_FUNC_CTX, openssl, "EVP_MD_CTX_init");	
		EVP_MD_CTX_destroy = GET_PROC(EVP_MD_FUNC_CTX, openssl, "EVP_MD_CTX_destroy");				
		
		EVP_DigestInit = GET_PROC(EVP_MD_FUNC_Init, openssl, "EVP_DigestInit");
		EVP_DigestUpdate = GET_PROC(EVP_MD_FUNC_Update, openssl, "EVP_DigestUpdate");	
		EVP_DigestFinal	= GET_PROC(EVP_MD_FUNC_Final, openssl, "EVP_DigestFinal");		
		
		HMAC = GET_PROC(EVP_FUNC_HMAC, openssl, "HMAC");	
		
	}
	
}

// ------------------------------------ crypto ------------------------------------

void SHA256( PA_PluginParameters params )
{
	
	PA_Handle			h;
	unsigned char *		md;
	unsigned int		mdlen = SHA256_HASH_SIZE;
	unsigned char str[SHA256_HASH_SIZE_FOR_HEX_C];
	
	h = PA_GetBlobHandleParameter( params, 1 );
	md = (unsigned char *)malloc( SHA256_HASH_SIZE );

	if(md){
		
		memset( md, 0x00, SHA256_HASH_SIZE );	

		EVP_MD_CTX* c = EVP_MD_CTX_create();
		EVP_MD_CTX_init(c);
		EVP_DigestInit(c, EVP_sha256());
		EVP_DigestUpdate(c, (unsigned char*)PA_LockHandle(h), PA_GetHandleSize(h));
		EVP_DigestFinal(c, md, &mdlen);	
		EVP_MD_CTX_destroy(c);
		
		PA_UnlockHandle(h);	
		SHA256_HASH_HEX(str,md);
		free(md);	
		
		PA_ReturnStringUTF8( params, str, SHA256_HASH_SIZE_FOR_HEX );
		
	}
	
}

void MD5( PA_PluginParameters params )
{
	
	PA_Handle			h;
	unsigned char *		md;
	unsigned int		mdlen = MD5_HASH_SIZE;	
	unsigned char str[MD5_HASH_SIZE_FOR_HEX_C];	
	
	h = PA_GetBlobHandleParameter( params, 1 );
	md = (unsigned char *)malloc( MD5_HASH_SIZE );
	
	if(md){
	
		memset( md, 0x00, MD5_HASH_SIZE );	
	
		EVP_MD_CTX* c = EVP_MD_CTX_create();
		EVP_MD_CTX_init(c);
		EVP_DigestInit(c, EVP_md5());
		EVP_DigestUpdate(c, (unsigned char*)PA_LockHandle(h), PA_GetHandleSize(h));
		EVP_DigestFinal(c, md, &mdlen);	
		EVP_MD_CTX_destroy(c);		
		
		PA_UnlockHandle(h);			
		MD5_HASH_HEX(str,md);
		free(md);
		
		PA_ReturnStringUTF8( params, str, MD5_HASH_SIZE_FOR_HEX );			

	}
	
}

void SHA1( PA_PluginParameters params )
{
	
	PA_Handle			h;
	unsigned char *		md;
	unsigned int		mdlen = SHA1_HASH_SIZE;		
	unsigned char str[SHA1_HASH_SIZE_FOR_HEX_C];
	
	h = PA_GetBlobHandleParameter( params, 1 );
	md = (unsigned char *)malloc( SHA1_HASH_SIZE );
	
	if(md){
		
		memset( md, 0x00, SHA1_HASH_SIZE );	
		
		EVP_MD_CTX* c = EVP_MD_CTX_create();
		EVP_MD_CTX_init(c);
		EVP_DigestInit(c, EVP_sha1());
		EVP_DigestUpdate(c, (unsigned char*)PA_LockHandle(h), PA_GetHandleSize(h));
		EVP_DigestFinal(c, md, &mdlen);	
		EVP_MD_CTX_destroy(c);		
		
		PA_UnlockHandle(h);			
		SHA1_HASH_HEX(str,md);
		free(md);
		
		PA_ReturnStringUTF8( params, str, SHA1_HASH_SIZE_FOR_HEX );			
		
	}
	
}

void SHA384( PA_PluginParameters params )
{
	
	PA_Handle			h;
	unsigned char *		md;
	unsigned int		mdlen = SHA384_HASH_SIZE;		
	unsigned char str[SHA384_HASH_SIZE_FOR_HEX_C];
	
	h = PA_GetBlobHandleParameter( params, 1 );
	md = (unsigned char *)malloc( SHA384_HASH_SIZE );
	
	if(md){
		
		memset( md, 0x00, SHA384_HASH_SIZE );	
		
		EVP_MD_CTX* c = EVP_MD_CTX_create();
		EVP_MD_CTX_init(c);
		EVP_DigestInit(c, EVP_sha384());
		EVP_DigestUpdate(c, (unsigned char*)PA_LockHandle(h), PA_GetHandleSize(h));
		EVP_DigestFinal(c, md, &mdlen);	
		EVP_MD_CTX_destroy(c);		
		
		PA_UnlockHandle(h);			
		SHA384_HASH_HEX(str,md);
		free(md);
		
		PA_ReturnStringUTF8( params, str, SHA384_HASH_SIZE_FOR_HEX );			
		
	}
		
}

void SHA512( PA_PluginParameters params )
{
	
	PA_Handle			h;
	unsigned char *		md;
	unsigned int		mdlen = SHA512_HASH_SIZE;		
	unsigned char str[SHA512_HASH_SIZE_FOR_HEX_C];
	
	h = PA_GetBlobHandleParameter( params, 1 );
	md = (unsigned char *)malloc( SHA512_HASH_SIZE );
	
	if(md){
		
		memset( md, 0x00, SHA512_HASH_SIZE );	
		
		EVP_MD_CTX* c = EVP_MD_CTX_create();
		EVP_MD_CTX_init(c);
		EVP_DigestInit(c, EVP_sha512());
		EVP_DigestUpdate(c, (unsigned char*)PA_LockHandle(h), PA_GetHandleSize(h));
		EVP_DigestFinal(c, md, &mdlen);	
		EVP_MD_CTX_destroy(c);		
		
		PA_UnlockHandle(h);			
		SHA512_HASH_HEX(str,md);
		free(md);
		
		PA_ReturnStringUTF8( params, str, SHA512_HASH_SIZE_FOR_HEX );			
		
	}
	
}

void HMACMD5( PA_PluginParameters params )
{
	
	PA_Handle			key,data;
	unsigned char *		md;
	unsigned int		mdlen = HMACMD5_HASH_SIZE;	
	unsigned char str[HMACMD5_HASH_SIZE_FOR_HEX_C];
	
	key = PA_GetBlobHandleParameter( params, 1 );
	data = PA_GetBlobHandleParameter( params, 2 );
	
	md = (unsigned char *)malloc( HMACMD5_HASH_SIZE );	
	
	if(md){
		
		memset( md, 0x00, HMACMD5_HASH_SIZE );	

		HMAC( EVP_md5(), (const void *)PA_LockHandle(key), (int)PA_GetHandleSize(key), (const unsigned char *)PA_LockHandle(data), (int)PA_GetHandleSize(data), md, &mdlen );
			
		PA_UnlockHandle(key);		
		PA_UnlockHandle(data);		
		
		HMACMD5_HASH_HEX(str,md);
		free(md);
		
		PA_ReturnStringUTF8( params, str, HMACMD5_HASH_SIZE_FOR_HEX );			
		
	}
	
}

void HMACSHA1( PA_PluginParameters params )
{
	
	PA_Handle			key,data;
	unsigned char *		md;
	unsigned int		mdlen = HMACSHA1_HASH_SIZE;	
	unsigned char str[HMACSHA1_HASH_SIZE_FOR_HEX_C];
	
	key = PA_GetBlobHandleParameter( params, 1 );
	data = PA_GetBlobHandleParameter( params, 2 );
	
	md = (unsigned char *)malloc( HMACSHA1_HASH_SIZE );	
	
	if(md){
		
		memset( md, 0x00, HMACSHA1_HASH_SIZE );	
		
		HMAC( EVP_sha1(), (const void *)PA_LockHandle(key), (int)PA_GetHandleSize(key), (const unsigned char *)PA_LockHandle(data), (int)PA_GetHandleSize(data), md, &mdlen );
		
		PA_UnlockHandle(key);		
		PA_UnlockHandle(data);		
		
		HMACSHA1_HASH_HEX(str,md);
		free(md);
		
		PA_ReturnStringUTF8( params, str, HMACSHA1_HASH_SIZE_FOR_HEX );			
		
	}
		
}

void HMACSHA256( PA_PluginParameters params )
{
	
	PA_Handle			key,data;
	unsigned char *		md;
	unsigned int		mdlen = HMACSHA256_HASH_SIZE;	
	unsigned char str[HMACSHA256_HASH_SIZE_FOR_HEX_C];
	
	key = PA_GetBlobHandleParameter( params, 1 );
	data = PA_GetBlobHandleParameter( params, 2 );
	
	md = (unsigned char *)malloc( HMACSHA256_HASH_SIZE );	
	
	if(md){
		
		memset( md, 0x00, HMACSHA256_HASH_SIZE );	
		
		HMAC( EVP_sha256(), (const void *)PA_LockHandle(key), (int)PA_GetHandleSize(key), (const unsigned char *)PA_LockHandle(data), (int)PA_GetHandleSize(data), md, &mdlen );
		
		PA_UnlockHandle(key);		
		PA_UnlockHandle(data);		
		
		HMACSHA256_HASH_HEX(str,md);
		free(md);
		
		PA_ReturnStringUTF8( params, str, HMACSHA256_HASH_SIZE_FOR_HEX );			
		
	}
	
}

void HMACSHA384( PA_PluginParameters params )
{
	
	PA_Handle			key,data;
	unsigned char *		md;
	unsigned int		mdlen = HMACSHA384_HASH_SIZE;	
	unsigned char str[HMACSHA384_HASH_SIZE_FOR_HEX_C];
	
	key = PA_GetBlobHandleParameter( params, 1 );
	data = PA_GetBlobHandleParameter( params, 2 );
	
	md = (unsigned char *)malloc( HMACSHA384_HASH_SIZE );	
	
	if(md){
		
		memset( md, 0x00, HMACSHA384_HASH_SIZE );	
		
		HMAC( EVP_sha384(), (const void *)PA_LockHandle(key), (int)PA_GetHandleSize(key), (const unsigned char *)PA_LockHandle(data), (int)PA_GetHandleSize(data), md, &mdlen );
		
		PA_UnlockHandle(key);		
		PA_UnlockHandle(data);		
		
		HMACSHA384_HASH_HEX(str,md);
		free(md);
		
		PA_ReturnStringUTF8( params, str, HMACSHA384_HASH_SIZE_FOR_HEX );			
		
	}	
	
}

void HMACSHA512( PA_PluginParameters params )
{
	
	PA_Handle			key,data;
	unsigned char *		md;
	unsigned int		mdlen = HMACSHA512_HASH_SIZE;	
	unsigned char str[HMACSHA512_HASH_SIZE_FOR_HEX_C];
	
	key = PA_GetBlobHandleParameter( params, 1 );
	data = PA_GetBlobHandleParameter( params, 2 );
	
	md = (unsigned char *)malloc( HMACSHA512_HASH_SIZE );	
	
	if(md){
		
		memset( md, 0x00, HMACSHA512_HASH_SIZE );	
		
		HMAC( EVP_sha512(), (const void *)PA_LockHandle(key), (int)PA_GetHandleSize(key), (const unsigned char *)PA_LockHandle(data), (int)PA_GetHandleSize(data), md, &mdlen );
		
		PA_UnlockHandle(key);		
		PA_UnlockHandle(data);		
		
		HMACSHA512_HASH_HEX(str,md);
		free(md);
		
		PA_ReturnStringUTF8( params, str, HMACSHA512_HASH_SIZE_FOR_HEX );			
		
	}
	
}