/* --------------------------------------------------------------------------------
 #
 #	4DPlugin.c
 #	source generated by 4D Plugin Wizard
 #	Project : 4D Cryptographic Hash
 #	author : miyako keisuke
 #	09/07/20
 #
 # --------------------------------------------------------------------------------*/

#include "4DPluginAPI.h"
#include "4DPlugin.h"
#if VERSIONMAC
#include <CommonCrypto/CommonHMAC.h>
#else
#include <windows.h>
#include <wincrypt.h>
#include <stdio.h>
#ifndef __FOURDPACKEX__
#define MAX_KEY_LENGTH 512
#endif
#pragma comment (lib, "Crypt32")
#endif

void PluginMain( long selector, PA_PluginParameters params )
{

	switch( selector )
	{

// --- Secure Hash Algorithm

		case 1 :
			SHA1( params );
			break;

		case 2 :
			SHA256( params );
			break;

		case 3 :
			SHA384( params );
			break;

		case 4 :
			SHA512( params );
			break;

		case 5 :
			HMACSHA512( params );
			break;

		case 6 :
			HMACSHA384( params );
			break;

		case 7 :
			HMACSHA256( params );
			break;

		case 8 :
			HMACSHA1( params );
			break;

// --- Message Digest Algorithm

		case 9 :
			HMACMD5( params );
			break;

		case 10 :
			MD5( params );
			break;

	}
}

// ----------------------------- Secure Hash Algorithm ----------------------------


void SHA1( PA_PluginParameters params )
{
	
	PA_Handle			data;
	char *				ustr;
	unsigned char *		md;
	char				str[41];
	const int			mdSize = 20;
		
	data = PA_GetBlobHandleParameter( params, 1 );
	
	if(data)
	{
	
		md = (unsigned char *)malloc( mdSize );
		
		if(md)
		{
#if VERSIONMAC	
			memset( md, 0x00, mdSize );
			
			unsigned char* srcData = (unsigned char *)PA_LockHandle(data);	
			
			if (srcData){
				CC_SHA1( srcData, PA_GetHandleSize(data), md );
				PA_UnlockHandle(data);
			}			
			
			sprintf( str, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
					md[ 0], md[ 1], md[ 2], md[ 3], md[ 4],
					md[ 5], md[ 6], md[ 7], md[ 8], md[ 9],
					md[10], md[11], md[12], md[13], md[14],
					md[15], md[16], md[17], md[18], md[19]				
					);
#else
			
			HCRYPTPROV	hCryptProv = 0;
			HCRYPTHASH	hHash = 0;
			DWORD       len = 20;
			unsigned char* srcData = (unsigned char *)PA_LockHandle(data);	

			memset( md, 0x00, len );
			
			
				if (srcData){

					if( !CryptAcquireContext( &hCryptProv, NULL, NULL, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT ) ) 
						CryptAcquireContext( &hCryptProv, NULL, NULL, PROV_RSA_FULL, CRYPT_NEWKEYSET ); 
					
					if(hCryptProv)
					{
						if( CryptCreateHash( hCryptProv, CALG_SHA1, 0, 0, &hHash ) ) 
							if( CryptHashData( hHash, srcData,  PA_GetHandleSize(data), 0 ) )
								CryptGetHashParam( hHash, HP_HASHVAL, md, &len, 0 );
						
						if(hHash)CryptDestroyHash(hHash);
						CryptReleaseContext(hCryptProv,0);
					}			
					
					PA_UnlockHandle(data);
				}			
			
				sprintf_s( str, 41, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
						md[ 0], md[ 1], md[ 2], md[ 3], md[ 4],
						md[ 5], md[ 6], md[ 7], md[ 8], md[ 9],
						md[10], md[11], md[12], md[13], md[14],
						md[15], md[16], md[17], md[18], md[19]				
						);
#endif			
			free(md);
			
			ustr = (char *)malloc( 82 );
			
			if(ustr)
			{
				memset( ustr, 0x00, 82 );
				PA_ConvertCharsetToCharset( str, 40, eVTC_UTF_8, ustr, 80, eVTC_UTF_16 );
				PA_ReturnString( params, (PA_Unichar *) ustr );	
				free( ustr );
			}
						
		}
	
	}
	
}

void SHA256( PA_PluginParameters params )
{
	
	PA_Handle			data;
	char *				ustr;
	unsigned char *		md;
	char				str[65];
	const int			mdSize = 32;
	
	data = PA_GetBlobHandleParameter( params, 1 );
	
	if(data)
	{
	
		md = (unsigned char *)malloc( 32 );
		
		if(md)
		{
#if VERSIONMAC
			memset( md, 0x00, mdSize );
			
			unsigned char* srcData = (unsigned char *)PA_LockHandle(data);	
			
			if (srcData){
				CC_SHA256( srcData, PA_GetHandleSize(data), md );
				PA_UnlockHandle(data);
			}			
			sprintf( str, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
					md[ 0], md[ 1], md[ 2], md[ 3], md[ 4],
					md[ 5], md[ 6], md[ 7], md[ 8], md[ 9],
					md[10], md[11], md[12], md[13], md[14],
					md[15], md[16], md[17], md[18], md[19],
					md[20], md[21], md[22], md[23], md[24],
					md[25], md[26], md[27], md[28], md[29],				
					md[30], md[31]			
					);
#else
		
			HCRYPTPROV	hCryptProv = 0;
			HCRYPTHASH	hHash = 0;
			DWORD       len = 32;
			unsigned char* srcData = (unsigned char *)PA_LockHandle(data);			

			memset( md, 0x00, len );
		
			if (srcData){

				if( !CryptAcquireContext( &hCryptProv, NULL, NULL, PROV_RSA_AES, CRYPT_VERIFYCONTEXT ) ) 
					CryptAcquireContext( &hCryptProv, NULL, NULL, PROV_RSA_AES, CRYPT_NEWKEYSET ); 
				
				if(hCryptProv)
				{
					if( CryptCreateHash( hCryptProv, CALG_SHA_256, 0, 0, &hHash ) ) 
						if( CryptHashData( hHash, (const BYTE *)*data,  PA_GetHandleSize(data), 0 ) )
							CryptGetHashParam( hHash, HP_HASHVAL, md, &len, 0 );
					
					if(hHash)CryptDestroyHash(hHash);
					CryptReleaseContext(hCryptProv,0);
				}
				
				PA_UnlockHandle(data);
			}
			
			sprintf_s( str, 65, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
					md[ 0], md[ 1], md[ 2], md[ 3], md[ 4],
					md[ 5], md[ 6], md[ 7], md[ 8], md[ 9],
					md[10], md[11], md[12], md[13], md[14],
					md[15], md[16], md[17], md[18], md[19],
					md[20], md[21], md[22], md[23], md[24],
					md[25], md[26], md[27], md[28], md[29],				
					md[30], md[31]			
					);			
#endif			
			free(md);
			
			ustr = (char *)malloc( 130 );
			
			if(ustr)
			{
				memset( ustr, 0x00, 130 );
				PA_ConvertCharsetToCharset( str, 64, eVTC_UTF_8, ustr, 128, eVTC_UTF_16 );
				PA_ReturnString( params, (PA_Unichar *) ustr );	
				free( ustr );
			}
			
		}
		
	}
	
}

void SHA384( PA_PluginParameters params )
{
	
	PA_Handle			data;
	char *				ustr;
	unsigned char *		md;
	char				str[97];
	const int			mdSize = 48;
	
	data = PA_GetBlobHandleParameter( params, 1 );
	
	if(data)
	{
	
		md = (unsigned char *)malloc( 48 );
		
		if(md)
		{
#if VERSIONMAC
			memset( md, 0x00, mdSize );
			unsigned char* srcData = (unsigned char *)PA_LockHandle(data);			
			if (srcData){
				CC_SHA384( srcData, PA_GetHandleSize(data), md );
				PA_UnlockHandle(data);
			}				
			sprintf( str, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
					md[ 0], md[ 1], md[ 2], md[ 3], md[ 4],
					md[ 5], md[ 6], md[ 7], md[ 8], md[ 9],
					md[10], md[11], md[12], md[13], md[14],
					md[15], md[16], md[17], md[18], md[19],
					md[20], md[21], md[22], md[23], md[24],
					md[25], md[26], md[27], md[28], md[29],				
					md[30], md[31], md[32], md[33], md[34],
					md[35], md[36], md[37], md[38], md[39],		
					md[40], md[41], md[42], md[43], md[44],	
					md[45], md[46], md[47]				
					);
#else
					
			HCRYPTPROV	hCryptProv = 0;
			HCRYPTHASH	hHash = 0;
			DWORD       len = 48;
			unsigned char* srcData = (unsigned char *)PA_LockHandle(data);	

			memset( md, 0x00, len );
			
			if (srcData){

				if( !CryptAcquireContext( &hCryptProv, NULL, NULL, PROV_RSA_AES, CRYPT_VERIFYCONTEXT ) ) 
					CryptAcquireContext( &hCryptProv, NULL, NULL, PROV_RSA_AES, CRYPT_NEWKEYSET ); 
				
				if(hCryptProv)
				{
					if( CryptCreateHash( hCryptProv, CALG_SHA_384, 0, 0, &hHash ) ) 
						if( CryptHashData( hHash, (const BYTE *)*data,  PA_GetHandleSize(data), 0 ) )
							CryptGetHashParam( hHash, HP_HASHVAL, md, &len, 0 );
					
					if(hHash)CryptDestroyHash(hHash);
					CryptReleaseContext(hCryptProv,0);
				}
				
				PA_UnlockHandle(data);
			}
		
			sprintf_s( str, 97, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
					md[ 0], md[ 1], md[ 2], md[ 3], md[ 4],
					md[ 5], md[ 6], md[ 7], md[ 8], md[ 9],
					md[10], md[11], md[12], md[13], md[14],
					md[15], md[16], md[17], md[18], md[19],
					md[20], md[21], md[22], md[23], md[24],
					md[25], md[26], md[27], md[28], md[29],				
					md[30], md[31], md[32], md[33], md[34],
					md[35], md[36], md[37], md[38], md[39],		
					md[40], md[41], md[42], md[43], md[44],	
					md[45], md[46], md[47]				
					);			
#endif				
			free(md);
			
			ustr = (char *)malloc( 194 );
			
			if(ustr)
			{
				memset( ustr, 0x00, 194 );
				PA_ConvertCharsetToCharset( str, 96, eVTC_UTF_8, ustr, 192, eVTC_UTF_16 );
				PA_ReturnString( params, (PA_Unichar *) ustr );	
				free( ustr );
			}
			
		}
	
	}
	
}

void SHA512( PA_PluginParameters params )
{
	
	PA_Handle			data;
	char *				ustr;
	unsigned char *		md;
	char				str[129];
	const int			mdSize = 64;
	
	data = PA_GetBlobHandleParameter( params, 1 );
	
	if(data)
	{

		md = (unsigned char *)malloc( 64 );
	
		if(md)
		{
#if VERSIONMAC
			memset( md, 0x00, mdSize );
			unsigned char* srcData = (unsigned char *)PA_LockHandle(data);			
			if (srcData){
				CC_SHA512( srcData, PA_GetHandleSize(data), md );
				PA_UnlockHandle(data);
			}				
			sprintf( str, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
					md[ 0], md[ 1], md[ 2], md[ 3], md[ 4],
					md[ 5], md[ 6], md[ 7], md[ 8], md[ 9],
					md[10], md[11], md[12], md[13], md[14],
					md[15], md[16], md[17], md[18], md[19],
					md[20], md[21], md[22], md[23], md[24],
					md[25], md[26], md[27], md[28], md[29],				
					md[30], md[31], md[32], md[33], md[34],
					md[35], md[36], md[37], md[38], md[39],		
					md[40], md[41], md[42], md[43], md[44],	
					md[45], md[46], md[47], md[48], md[49],		
					md[50], md[51], md[52], md[53], md[54],	
					md[55], md[56], md[57], md[58], md[59],	
					md[60], md[61], md[62], md[63]				
					);
#else
						
			HCRYPTPROV	hCryptProv = 0;
			HCRYPTHASH	hHash = 0;
			DWORD       len = 64;
			unsigned char* srcData = (unsigned char *)PA_LockHandle(data);		

			memset( md, 0x00, len );
						
			if (srcData){

				if( !CryptAcquireContext( &hCryptProv, NULL, NULL, PROV_RSA_AES, CRYPT_VERIFYCONTEXT ) ) 
					CryptAcquireContext( &hCryptProv, NULL, NULL, PROV_RSA_AES, CRYPT_NEWKEYSET ); 
				
				if(hCryptProv)
				{
					if( CryptCreateHash( hCryptProv, CALG_SHA_512, 0, 0, &hHash ) ) 
						if( CryptHashData( hHash, (const BYTE *)*data,  PA_GetHandleSize(data), 0 ) )
							CryptGetHashParam( hHash, HP_HASHVAL, md, &len, 0 );
					
					if(hHash)CryptDestroyHash(hHash);
					CryptReleaseContext(hCryptProv,0);
				}
				
				PA_UnlockHandle(data);
			}
		
			sprintf_s( str, 129, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
					md[ 0], md[ 1], md[ 2], md[ 3], md[ 4],
					md[ 5], md[ 6], md[ 7], md[ 8], md[ 9],
					md[10], md[11], md[12], md[13], md[14],
					md[15], md[16], md[17], md[18], md[19],
					md[20], md[21], md[22], md[23], md[24],
					md[25], md[26], md[27], md[28], md[29],				
					md[30], md[31], md[32], md[33], md[34],
					md[35], md[36], md[37], md[38], md[39],		
					md[40], md[41], md[42], md[43], md[44],	
					md[45], md[46], md[47], md[48], md[49],		
					md[50], md[51], md[52], md[53], md[54],	
					md[55], md[56], md[57], md[58], md[59],	
					md[60], md[61], md[62], md[63]				
					);				
#endif			
			free(md);
			
			ustr = (char *)malloc( 258 );
			
			if(ustr)
			{
				memset( ustr, 0x00, 258 );
				PA_ConvertCharsetToCharset( str, 128, eVTC_UTF_8, ustr, 256, eVTC_UTF_16 );
				PA_ReturnString( params, (PA_Unichar *) ustr );	
				free( ustr );
			}
			
		}
		
	}
	
}

void HMACSHA512( PA_PluginParameters params )
{
	
	PA_Handle			key, data;
	char *				ustr;
	unsigned char *		md;
	char				str[129];
	const int			mdSize = 64;
	
	key = PA_GetBlobHandleParameter( params, 1 );
	data = PA_GetBlobHandleParameter( params, 2 );
	
	if(key)
	{
		if(data)
		{
		
			md = (unsigned char *)malloc( 64 );
			
			if(md)
			{
#if VERSIONMAC
				memset( md, 0x00, mdSize );
				unsigned char* srcData = (unsigned char *)PA_LockHandle(data);	
				unsigned char* keyData = (unsigned char *)PA_LockHandle(key);					
				if (srcData){
					if (keyData){					
						CCHmac( kCCHmacAlgSHA512, keyData, PA_GetHandleSize(key), srcData, PA_GetHandleSize(data), md );
						PA_UnlockHandle(key);	
					}
					PA_UnlockHandle(data);
				}				
				sprintf( str, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
						md[ 0], md[ 1], md[ 2], md[ 3], md[ 4],
						md[ 5], md[ 6], md[ 7], md[ 8], md[ 9],
						md[10], md[11], md[12], md[13], md[14],
						md[15], md[16], md[17], md[18], md[19],
						md[20], md[21], md[22], md[23], md[24],
						md[25], md[26], md[27], md[28], md[29],				
						md[30], md[31], md[32], md[33], md[34],
						md[35], md[36], md[37], md[38], md[39],		
						md[40], md[41], md[42], md[43], md[44],	
						md[45], md[46], md[47], md[48], md[49],		
						md[50], md[51], md[52], md[53], md[54],	
						md[55], md[56], md[57], md[58], md[59],	
						md[60], md[61], md[62], md[63]				
						);
#else
						
				HCRYPTPROV	hCryptProv = 0;
				HCRYPTHASH	hHash = 0;
				HCRYPTKEY	hKey = 0;
				DWORD       len = 64;
				HMAC_INFO	hi;
				unsigned char* srcData = (unsigned char *)PA_LockHandle(data);	
				unsigned char* keyData = (unsigned char *)PA_LockHandle(key);	
				
				struct {
					BLOBHEADER hdr;
					DWORD      len;
					BYTE       key[MAX_KEY_LENGTH];
				} KEYBLOB;
				
				ZeroMemory(&KEYBLOB, sizeof(KEYBLOB));
				ZeroMemory(&hi, sizeof(hi));
				
				KEYBLOB.hdr.bType    = PLAINTEXTKEYBLOB;
				KEYBLOB.hdr.bVersion = CUR_BLOB_VERSION;
				KEYBLOB.hdr.reserved = 0;
				KEYBLOB.hdr.aiKeyAlg = CALG_RC2;
				KEYBLOB.len          = PA_GetHandleSize(key);
								
				if (srcData){
					if (keyData){					

						if(MAX_KEY_LENGTH<PA_GetHandleSize(key))
						{
							memcpy(KEYBLOB.key, *key, MAX_KEY_LENGTH);						
						}else{
							memcpy(KEYBLOB.key, *key, PA_GetHandleSize(key));				
						}
						
						memset( md, 0x00, len );
						
						hi.HashAlgid = CALG_SHA_512;
						
						if( !CryptAcquireContext( &hCryptProv, NULL, NULL, PROV_RSA_AES, CRYPT_VERIFYCONTEXT ) ) 
							CryptAcquireContext( &hCryptProv, NULL, NULL, PROV_RSA_AES, CRYPT_NEWKEYSET ); 
						
						if(hCryptProv)
						{
							
							if( CryptImportKey(hCryptProv, (BYTE *) &KEYBLOB, sizeof(KEYBLOB), 0, CRYPT_IPSEC_HMAC_KEY, &hKey ) )
								
								
								if( CryptCreateHash( hCryptProv, CALG_HMAC, hKey, 0, &hHash ) ) 
									if(CryptSetHashParam( hHash, HP_HMAC_INFO, (BYTE *)&hi, 0 ) )
										if( CryptHashData( hHash, (const BYTE *)*data,  PA_GetHandleSize(data), 0 ) )
											CryptGetHashParam( hHash, HP_HASHVAL, md, &len, 0 );
							
							if(hHash)CryptDestroyHash(hHash);
							if(hKey)CryptDestroyKey(hKey);					
							CryptReleaseContext(hCryptProv,0);
						}
						
						PA_UnlockHandle(key);	
					}
					PA_UnlockHandle(data);
				}
					
				sprintf_s( str, 129, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
						md[ 0], md[ 1], md[ 2], md[ 3], md[ 4],
						md[ 5], md[ 6], md[ 7], md[ 8], md[ 9],
						md[10], md[11], md[12], md[13], md[14],
						md[15], md[16], md[17], md[18], md[19],
						md[20], md[21], md[22], md[23], md[24],
						md[25], md[26], md[27], md[28], md[29],				
						md[30], md[31], md[32], md[33], md[34],
						md[35], md[36], md[37], md[38], md[39],		
						md[40], md[41], md[42], md[43], md[44],	
						md[45], md[46], md[47], md[48], md[49],		
						md[50], md[51], md[52], md[53], md[54],	
						md[55], md[56], md[57], md[58], md[59],	
						md[60], md[61], md[62], md[63]				
						);				
#endif				
				free(md);
				
				ustr = (char *)malloc( 258 );
				
				if(ustr)
				{
					memset( ustr, 0x00, 258 );
					PA_ConvertCharsetToCharset( str, 128, eVTC_UTF_8, ustr, 256, eVTC_UTF_16 );
					PA_ReturnString( params, (PA_Unichar *) ustr );	
					free( ustr );
				}
				
			}
			
		}
		
	}
	
}

void HMACSHA384( PA_PluginParameters params )
{

	PA_Handle			key, data;
	char *				ustr;
	unsigned char *		md;
	char				str[97];
	const int			mdSize = 48;
	
	key = PA_GetBlobHandleParameter( params, 1 );
	data = PA_GetBlobHandleParameter( params, 2 );
	
	if(key)
	{
		if(data)
		{	
		
			md = (unsigned char *)malloc( 48 );
			
			if(md)
			{
#if VERSIONMAC
				memset( md, 0x00, mdSize );
				unsigned char* srcData = (unsigned char *)PA_LockHandle(data);	
				unsigned char* keyData = (unsigned char *)PA_LockHandle(key);					
				if (srcData){
					if (keyData){					
						CCHmac( kCCHmacAlgSHA384, keyData, PA_GetHandleSize(key), srcData, PA_GetHandleSize(data), md );
						PA_UnlockHandle(key);	
					}
					PA_UnlockHandle(data);
				}				
				sprintf( str, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
						md[ 0], md[ 1], md[ 2], md[ 3], md[ 4],
						md[ 5], md[ 6], md[ 7], md[ 8], md[ 9],
						md[10], md[11], md[12], md[13], md[14],
						md[15], md[16], md[17], md[18], md[19],
						md[20], md[21], md[22], md[23], md[24],
						md[25], md[26], md[27], md[28], md[29],				
						md[30], md[31], md[32], md[33], md[34],
						md[35], md[36], md[37], md[38], md[39],		
						md[40], md[41], md[42], md[43], md[44],	
						md[45], md[46], md[47]				
						);
#else
							
				HCRYPTPROV	hCryptProv = 0;
				HCRYPTHASH	hHash = 0;
				HCRYPTKEY	hKey = 0;
				DWORD       len = 48;
				HMAC_INFO	hi;
				unsigned char* srcData = (unsigned char *)PA_LockHandle(data);	
				unsigned char* keyData = (unsigned char *)PA_LockHandle(key);					
				
				struct {
					BLOBHEADER hdr;
					DWORD      len;
					BYTE       key[MAX_KEY_LENGTH];
				} KEYBLOB;
				
				ZeroMemory(&KEYBLOB, sizeof(KEYBLOB));
				ZeroMemory(&hi, sizeof(hi));
				
				KEYBLOB.hdr.bType    = PLAINTEXTKEYBLOB;
				KEYBLOB.hdr.bVersion = CUR_BLOB_VERSION;
				KEYBLOB.hdr.reserved = 0;
				KEYBLOB.hdr.aiKeyAlg = CALG_RC2;
				KEYBLOB.len          = PA_GetHandleSize(key);
				
					if (srcData){
					if (keyData){					
	
						if(MAX_KEY_LENGTH<PA_GetHandleSize(key))
						{
							memcpy(KEYBLOB.key, *key, MAX_KEY_LENGTH);						
						}else{
							memcpy(KEYBLOB.key, *key, PA_GetHandleSize(key));				
						}
						
						memset( md, 0x00, len );
						
						hi.HashAlgid = CALG_SHA_384;
						
						if( !CryptAcquireContext( &hCryptProv, NULL, NULL, PROV_RSA_AES, CRYPT_VERIFYCONTEXT ) ) 
							CryptAcquireContext( &hCryptProv, NULL, NULL, PROV_RSA_AES, CRYPT_NEWKEYSET ); 
						
						if(hCryptProv)
						{
							
							if( CryptImportKey(hCryptProv, (BYTE *) &KEYBLOB, sizeof(KEYBLOB), 0, CRYPT_IPSEC_HMAC_KEY, &hKey ) )
								
								
								if( CryptCreateHash( hCryptProv, CALG_HMAC, hKey, 0, &hHash ) ) 
									if(CryptSetHashParam( hHash, HP_HMAC_INFO, (BYTE *)&hi, 0 ) )
										if( CryptHashData( hHash, (const BYTE *)*data,  PA_GetHandleSize(data), 0 ) )
											CryptGetHashParam( hHash, HP_HASHVAL, md, &len, 0 );
							
							if(hHash)CryptDestroyHash(hHash);
							if(hKey)CryptDestroyKey(hKey);					
							CryptReleaseContext(hCryptProv,0);
						}
						
						PA_UnlockHandle(key);	
					}
					PA_UnlockHandle(data);
				}
				
				sprintf_s( str, 97, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
						md[ 0], md[ 1], md[ 2], md[ 3], md[ 4],
						md[ 5], md[ 6], md[ 7], md[ 8], md[ 9],
						md[10], md[11], md[12], md[13], md[14],
						md[15], md[16], md[17], md[18], md[19],
						md[20], md[21], md[22], md[23], md[24],
						md[25], md[26], md[27], md[28], md[29],				
						md[30], md[31], md[32], md[33], md[34],
						md[35], md[36], md[37], md[38], md[39],		
						md[40], md[41], md[42], md[43], md[44],	
						md[45], md[46], md[47]				
						);			
#endif				
				free(md);
				
				ustr = (char *)malloc( 194 );
				
				if(ustr)
				{
					memset( ustr, 0x00, 194 );
					PA_ConvertCharsetToCharset( str, 96, eVTC_UTF_8, ustr, 192, eVTC_UTF_16 );
					PA_ReturnString( params, (PA_Unichar *) ustr );	
					free( ustr );
				}
				
			}
		
		}
		
	}
	
}

void HMACSHA256( PA_PluginParameters params )
{
	
	PA_Handle			key, data;
	char *				ustr;
	unsigned char *		md;
	char				str[65];
	const int			mdSize = 32;
	
	key = PA_GetBlobHandleParameter( params, 1 );
	data = PA_GetBlobHandleParameter( params, 2 );
	
	if(key)
	{
		if(data)
		{	
		
			md = (unsigned char *)malloc( 32 );
			
			if(md)
			{
#if VERSIONMAC
				memset( md, 0x00, mdSize );
				unsigned char* srcData = (unsigned char *)PA_LockHandle(data);	
				unsigned char* keyData = (unsigned char *)PA_LockHandle(key);					
				if (srcData){
					if (keyData){					
						CCHmac( kCCHmacAlgSHA256, keyData, PA_GetHandleSize(key), srcData, PA_GetHandleSize(data), md );
						PA_UnlockHandle(key);	
					}
					PA_UnlockHandle(data);
				}				
				sprintf( str, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
						md[ 0], md[ 1], md[ 2], md[ 3], md[ 4],
						md[ 5], md[ 6], md[ 7], md[ 8], md[ 9],
						md[10], md[11], md[12], md[13], md[14],
						md[15], md[16], md[17], md[18], md[19],
						md[20], md[21], md[22], md[23], md[24],
						md[25], md[26], md[27], md[28], md[29],				
						md[30], md[31]			
						);
#else
							
				HCRYPTPROV	hCryptProv = 0;
				HCRYPTHASH	hHash = 0;
				HCRYPTKEY	hKey = 0;
				DWORD       len = 32;
				HMAC_INFO	hi;
				unsigned char* srcData = (unsigned char *)PA_LockHandle(data);	
				unsigned char* keyData = (unsigned char *)PA_LockHandle(key);					
				
				struct {
					BLOBHEADER hdr;
					DWORD      len;
					BYTE       key[MAX_KEY_LENGTH];
				} KEYBLOB;
				
				ZeroMemory(&KEYBLOB, sizeof(KEYBLOB));
				ZeroMemory(&hi, sizeof(hi));
				
				KEYBLOB.hdr.bType    = PLAINTEXTKEYBLOB;
				KEYBLOB.hdr.bVersion = CUR_BLOB_VERSION;
				KEYBLOB.hdr.reserved = 0;
				KEYBLOB.hdr.aiKeyAlg = CALG_RC2;
				KEYBLOB.len          = PA_GetHandleSize(key);
				
								if (srcData){
					if (keyData){					

						if(MAX_KEY_LENGTH<PA_GetHandleSize(key))
						{
							memcpy(KEYBLOB.key, *key, MAX_KEY_LENGTH);						
						}else{
							memcpy(KEYBLOB.key, *key, PA_GetHandleSize(key));				
						}
						
						memset( md, 0x00, len );
						
						hi.HashAlgid = CALG_SHA_256;
						
						if( !CryptAcquireContext( &hCryptProv, NULL, NULL, PROV_RSA_AES, CRYPT_VERIFYCONTEXT ) ) 
							CryptAcquireContext( &hCryptProv, NULL, NULL, PROV_RSA_AES, CRYPT_NEWKEYSET ); 
						
						if(hCryptProv)
						{
							
							if( CryptImportKey(hCryptProv, (BYTE *) &KEYBLOB, sizeof(KEYBLOB), 0, CRYPT_IPSEC_HMAC_KEY, &hKey ) )
								
								
								if( CryptCreateHash( hCryptProv, CALG_HMAC, hKey, 0, &hHash ) ) 
									if(CryptSetHashParam( hHash, HP_HMAC_INFO, (BYTE *)&hi, 0 ) )
										if( CryptHashData( hHash, (const BYTE *)*data,  PA_GetHandleSize(data), 0 ) )
											CryptGetHashParam( hHash, HP_HASHVAL, md, &len, 0 );
							
							if(hHash)CryptDestroyHash(hHash);
							if(hKey)CryptDestroyKey(hKey);					
							CryptReleaseContext(hCryptProv,0);
						}
												
						PA_UnlockHandle(key);	
					}
					PA_UnlockHandle(data);
				}	
				
				sprintf_s( str, 65, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
						md[ 0], md[ 1], md[ 2], md[ 3], md[ 4],
						md[ 5], md[ 6], md[ 7], md[ 8], md[ 9],
						md[10], md[11], md[12], md[13], md[14],
						md[15], md[16], md[17], md[18], md[19],
						md[20], md[21], md[22], md[23], md[24],
						md[25], md[26], md[27], md[28], md[29],				
						md[30], md[31]			
						);
#endif				
				free(md);
				
				ustr = (char *)malloc( 130 );
				
				if(ustr)
				{
					memset( ustr, 0x00, 130 );
					PA_ConvertCharsetToCharset( str, 64, eVTC_UTF_8, ustr, 128, eVTC_UTF_16 );
					PA_ReturnString( params, (PA_Unichar *) ustr );	
					free( ustr );
				}
				
			}
		
		}
		
	}

}

void HMACSHA1( PA_PluginParameters params )
{
	
	PA_Handle			key, data;
	char *				ustr;
	unsigned char *		md;
	char				str[41];
	const int			mdSize = 20;
	
	key = PA_GetBlobHandleParameter( params, 1 );
	data = PA_GetBlobHandleParameter( params, 2 );
	
	if(key)
	{
		if(data)
		{	
		
			md = (unsigned char *)malloc( 20 );
			
			if(md)
			{
#if VERSIONMAC
				memset( md, 0x00, mdSize );
				unsigned char* srcData = (unsigned char *)PA_LockHandle(data);	
				unsigned char* keyData = (unsigned char *)PA_LockHandle(key);					
				if (srcData){
					if (keyData){					
						CCHmac( kCCHmacAlgSHA1, keyData, PA_GetHandleSize(key), srcData, PA_GetHandleSize(data), md );
						PA_UnlockHandle(key);	
					}
					PA_UnlockHandle(data);
				}					
				sprintf( str, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
						md[ 0], md[ 1], md[ 2], md[ 3], md[ 4],
						md[ 5], md[ 6], md[ 7], md[ 8], md[ 9],
						md[10], md[11], md[12], md[13], md[14],
						md[15], md[16], md[17], md[18], md[19]				
						);
#else
						
				HCRYPTPROV	hCryptProv = 0;
				HCRYPTHASH	hHash = 0;
				HCRYPTKEY	hKey = 0;
				DWORD       len = 20;
				HMAC_INFO	hi;
				unsigned char* srcData = (unsigned char *)PA_LockHandle(data);	
				unsigned char* keyData = (unsigned char *)PA_LockHandle(key);					
				
				struct {
					BLOBHEADER hdr;
					DWORD      len;
					BYTE       key[MAX_KEY_LENGTH];
				} KEYBLOB;
				
				ZeroMemory(&KEYBLOB, sizeof(KEYBLOB));
				ZeroMemory(&hi, sizeof(hi));
				
				KEYBLOB.hdr.bType    = PLAINTEXTKEYBLOB;
				KEYBLOB.hdr.bVersion = CUR_BLOB_VERSION;
				KEYBLOB.hdr.reserved = 0;
				KEYBLOB.hdr.aiKeyAlg = CALG_RC2;
				KEYBLOB.len          = PA_GetHandleSize(key);
				
				if (srcData){
					if (keyData){					
	
						if(MAX_KEY_LENGTH<PA_GetHandleSize(key))
						{
							memcpy(KEYBLOB.key, *key, MAX_KEY_LENGTH);						
						}else{
							memcpy(KEYBLOB.key, *key, PA_GetHandleSize(key));				
						}
						
						memset( md, 0x00, len );
						
						hi.HashAlgid = CALG_SHA1;
						
						if( !CryptAcquireContext( &hCryptProv, NULL, NULL, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT ) ) 
							CryptAcquireContext( &hCryptProv, NULL, NULL, PROV_RSA_FULL, CRYPT_NEWKEYSET ); 
						
						if(hCryptProv)
						{
							
							if( CryptImportKey(hCryptProv, (BYTE *) &KEYBLOB, sizeof(KEYBLOB), 0, CRYPT_IPSEC_HMAC_KEY, &hKey ) )
								
								
								if( CryptCreateHash( hCryptProv, CALG_HMAC, hKey, 0, &hHash ) ) 
									if(CryptSetHashParam( hHash, HP_HMAC_INFO, (BYTE *)&hi, 0 ) )
										if( CryptHashData( hHash, (const BYTE *)*data,  PA_GetHandleSize(data), 0 ) )
											CryptGetHashParam( hHash, HP_HASHVAL, md, &len, 0 );
							
							if(hHash)CryptDestroyHash(hHash);
							if(hKey)CryptDestroyKey(hKey);					
							CryptReleaseContext(hCryptProv,0);
						}
						
						PA_UnlockHandle(key);	
					}
					PA_UnlockHandle(data);
				}
								
				sprintf_s( str, 41, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
						md[ 0], md[ 1], md[ 2], md[ 3], md[ 4],
						md[ 5], md[ 6], md[ 7], md[ 8], md[ 9],
						md[10], md[11], md[12], md[13], md[14],
						md[15], md[16], md[17], md[18], md[19]				
						);
#endif								
				free(md);
				
				ustr = (char *)malloc( 82 );
				
				if(ustr)
				{
					memset( ustr, 0x00, 82 );
					PA_ConvertCharsetToCharset( str, 40, eVTC_UTF_8, ustr, 80, eVTC_UTF_16 );
					PA_ReturnString( params, (PA_Unichar *) ustr );	
					free( ustr );
				}
				
			}
			
		}
		
	}
	
}

// --------------------------- Message Digest Algorithm ---------------------------


void HMACMD5( PA_PluginParameters params )
{
	
	PA_Handle			key, data;
	char *				ustr;
	unsigned char *		md;
	char				str[33];
	const int			mdSize = 16;
	
	key = PA_GetBlobHandleParameter( params, 1 );
	data = PA_GetBlobHandleParameter( params, 2 );
	
	if(key)
	{
		if(data)
		{	
		
			md = (unsigned char *)malloc( 16 );
			
			if(md)
			{
#if VERSIONMAC	
				memset( md, 0x00, mdSize );
				unsigned char* srcData = (unsigned char *)PA_LockHandle(data);	
				unsigned char* keyData = (unsigned char *)PA_LockHandle(key);					
				if (srcData){
					if (keyData){					
						CCHmac( kCCHmacAlgMD5, keyData, PA_GetHandleSize(key), srcData, PA_GetHandleSize(data), md );
						PA_UnlockHandle(key);	
					}
					PA_UnlockHandle(data);
				}					
				sprintf( str, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
						md[ 0], md[ 1], md[ 2], md[ 3], md[ 4],
						md[ 5], md[ 6], md[ 7], md[ 8], md[ 9],
						md[10], md[11], md[12], md[13], md[14],
						md[15]	
						);
#else
							
				HCRYPTPROV	hCryptProv = 0;
				HCRYPTHASH	hHash = 0;
				HCRYPTKEY	hKey = 0;
				DWORD       len = 16;
				HMAC_INFO	hi;
				unsigned char* srcData = (unsigned char *)PA_LockHandle(data);	
				unsigned char* keyData = (unsigned char *)PA_LockHandle(key);					

				struct {
					BLOBHEADER hdr;
					DWORD      len;
					BYTE       key[MAX_KEY_LENGTH];
					} KEYBLOB;
				
				ZeroMemory(&KEYBLOB, sizeof(KEYBLOB));
				ZeroMemory(&hi, sizeof(hi));

				KEYBLOB.hdr.bType    = PLAINTEXTKEYBLOB;
				KEYBLOB.hdr.bVersion = CUR_BLOB_VERSION;
				KEYBLOB.hdr.reserved = 0;
				KEYBLOB.hdr.aiKeyAlg = CALG_RC2;
				KEYBLOB.len          = PA_GetHandleSize(key);
				
				if (srcData){
					if (keyData){					
	
						if(MAX_KEY_LENGTH<PA_GetHandleSize(key))
						{
							memcpy(KEYBLOB.key, *key, MAX_KEY_LENGTH);						
						}else{
							memcpy(KEYBLOB.key, *key, PA_GetHandleSize(key));				
						}	
						
						memset( md, 0x00, len );
						
						hi.HashAlgid = CALG_MD5;
						
						if( !CryptAcquireContext( &hCryptProv, NULL, NULL, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT ) ) 
							CryptAcquireContext( &hCryptProv, NULL, NULL, PROV_RSA_FULL, CRYPT_NEWKEYSET ); 
						
						if(hCryptProv)
						{
							
							if( CryptImportKey(hCryptProv, (BYTE *) &KEYBLOB, sizeof(KEYBLOB), 0, CRYPT_IPSEC_HMAC_KEY, &hKey ) )
								
								if( CryptCreateHash( hCryptProv, CALG_HMAC, hKey, 0, &hHash ) ) 
									if(CryptSetHashParam( hHash, HP_HMAC_INFO, (BYTE *)&hi, 0 ) )
										if( CryptHashData( hHash, (const BYTE *)*data,  PA_GetHandleSize(data), 0 ) )
											CryptGetHashParam( hHash, HP_HASHVAL, md, &len, 0 );
							
							if(hHash)CryptDestroyHash(hHash);
							if(hKey)CryptDestroyKey(hKey);					
							CryptReleaseContext(hCryptProv,0);
						}
						
						PA_UnlockHandle(key);	
					}
					PA_UnlockHandle(data);
				}
				
				sprintf_s( str, 33, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
						  md[ 0], md[ 1], md[ 2], md[ 3], md[ 4],
						  md[ 5], md[ 6], md[ 7], md[ 8], md[ 9],
						  md[10], md[11], md[12], md[13], md[14],
						  md[15]	
						  );				
#endif							
				free(md);
				
				ustr = (char *)malloc( 66 );
				
				if(ustr)
				{
					memset( ustr, 0x00, 66 );
					PA_ConvertCharsetToCharset( str, 32, eVTC_UTF_8, ustr, 64, eVTC_UTF_16 );
					PA_ReturnString( params, (PA_Unichar *) ustr );	
					free( ustr );
				}
				
			}	
			
		}
		
	}
	
}

void MD5( PA_PluginParameters params )
{
	
	PA_Handle			data;
	char *				ustr;
	unsigned char *		md;
	char				str[33];
	const int			mdSize = 16;
	
	data = PA_GetBlobHandleParameter( params, 1 );
	
	if(data)
	{
		
		md = (unsigned char *)malloc( 16 );
		
		if(md)
		{
#if VERSIONMAC	
			memset( md, 0x00, mdSize );
			unsigned char* srcData = (unsigned char *)PA_LockHandle(data);			
			if (srcData){
				CC_MD5( srcData, PA_GetHandleSize(data), md );
				PA_UnlockHandle(data);
			}				
			sprintf( str, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
					md[ 0], md[ 1], md[ 2], md[ 3], md[ 4],
					md[ 5], md[ 6], md[ 7], md[ 8], md[ 9],
					md[10], md[11], md[12], md[13], md[14],
					md[15]	
					);
#else
						
			HCRYPTPROV	hCryptProv = 0;
			HCRYPTHASH	hHash = 0;
			DWORD       len = 16;
			unsigned char* srcData = (unsigned char *)PA_LockHandle(data);			

			memset( md, 0x00, len );
			
			if (srcData){

				if( !CryptAcquireContext( &hCryptProv, NULL, NULL, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT ) ) 
					CryptAcquireContext( &hCryptProv, NULL, NULL, PROV_RSA_FULL, CRYPT_NEWKEYSET ); 
				
				if(hCryptProv)
				{
					if( CryptCreateHash( hCryptProv, CALG_MD5, 0, 0, &hHash ) ) 
						if( CryptHashData( hHash, (const BYTE *)*data,  PA_GetHandleSize(data), 0 ) )
							CryptGetHashParam( hHash, HP_HASHVAL, md, &len, 0 );
					
					if(hHash)CryptDestroyHash(hHash);
					CryptReleaseContext(hCryptProv,0);
				}
				
				PA_UnlockHandle(data);
			}	
				
			sprintf_s( str, 33, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
					md[ 0], md[ 1], md[ 2], md[ 3], md[ 4],
					md[ 5], md[ 6], md[ 7], md[ 8], md[ 9],
					md[10], md[11], md[12], md[13], md[14],
					md[15]	
					);		
#endif			
			free(md);
	
			ustr = (char *)malloc( 66 );
			
			if(ustr)
			{
				memset( ustr, 0x00, 66 );
				PA_ConvertCharsetToCharset( str, 32, eVTC_UTF_8, ustr, 64, eVTC_UTF_16 );
				PA_ReturnString( params, (PA_Unichar *) ustr );	
				free( ustr );
			}

		}
		
	}
	
}