<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns="urn:schemas-microsoft-com:office:spreadsheet"
    xmlns:o="urn:schemas-microsoft-com:office:office"
    xmlns:x="urn:schemas-microsoft-com:office:excel" 
    xmlns:html="http://www.w3.org/TR/REC-html40" 
    xmlns:exsl4D="http://www.4D.com"
    xmlns:ss="urn:schemas-microsoft-com:office:spreadsheet">
<xsl:param name="default_font_name" />
<xsl:param name="default_font_charset" />
<xsl:param name="default_font_size" />
<xsl:param name="default_column_width" />
<xsl:param name="default_row_height" />
<xsl:param name="expanded_column_count" />
<xsl:param name="header_rows_count" />
<xsl:param name="header_style_color" />
<xsl:param name="header_style_pattern" />
<xsl:param name="invisible_table_style_pattern" />
<xsl:param name="invisible_field_style_pattern" />
<xsl:param name="header_style_font_color" />
<xsl:param name="header_style_font_bold" />
<xsl:output method="xml" indent="yes" />
<xsl:template match="/">
<Workbook>
<Styles>
<Style ss:ID="Default" ss:Name="Normal">
<Font ss:FontName="{$default_font_name}" x:CharSet="{$default_font_charset}" ss:Size="{$default_font_size}" />
</Style>
<Style ss:ID="s22">
<NumberFormat ss:Format="Short Date" />
</Style>
<Style ss:ID="s23">
<NumberFormat ss:Format="Short Time" />
</Style>
<Style ss:ID="s24">
<Alignment ss:WrapText="1" />
</Style>
<Style ss:ID="header_style">
<Font ss:Color="{$header_style_font_color}" ss:Bold="{$header_style_font_bold}" />
<Interior ss:Color="{$header_style_color}" ss:Pattern="{$header_style_pattern}"/>
</Style>
 <xsl:apply-templates select="//editor_table_info" />
 <xsl:apply-templates select="//editor_field_info" />
</Styles>
<Worksheet ss:Name="Sheet1">
<Table ss:ExpandedColumnCount="{$expanded_column_count}" ss:ExpandedRowCount="{count(//field) + count(/base/schema/table) + $header_rows_count}" x:FullColumns="1"
 x:FullRows="1" ss:DefaultColumnWidth="{$default_column_width}" ss:DefaultRowHeight="{$default_row_height}">
<Row><Cell><Data ss:Type="String">name:</Data></Cell>
<Cell><Data ss:Type="String"><xsl:value-of select="/base/@name" />
</Data></Cell></Row>
<Row><Cell><Data ss:Type="String">uuid:</Data></Cell>
<Cell><Data ss:Type="String"><xsl:value-of select="/base/@uuid" />
</Data></Cell></Row>
<Row><Cell><Data ss:Type="String">locale:</Data></Cell>
<Cell><Data ss:Type="String"><xsl:value-of select="/base/@collation_locale" />
</Data></Cell></Row>
<Row><Cell><Data ss:Type="String">tables:</Data></Cell>
<Cell><Data ss:Type="Number"><xsl:value-of select="count(//table)" />
</Data></Cell></Row>
<Row><Cell><Data ss:Type="String">fields:</Data></Cell>
<Cell><Data ss:Type="Number"><xsl:value-of select="count(//field)" />
</Data></Cell></Row>
<Row><Cell><Data ss:Type="String">indices:</Data></Cell>
<Cell><Data ss:Type="Number"><xsl:value-of select="count(//index)" />
</Data></Cell></Row>
<Row />
<Row>
<Cell ss:StyleID="header_style"><Data ss:Type="String">table name</Data></Cell>
<Cell ss:StyleID="header_style"><Data ss:Type="String">table uuid</Data></Cell>
<Cell ss:StyleID="header_style"><Data ss:Type="String">field name</Data></Cell>
<Cell ss:StyleID="header_style"><Data ss:Type="String">field uuid</Data></Cell>
<Cell ss:StyleID="header_style"><Data ss:Type="String">field type</Data></Cell>
<Cell ss:StyleID="header_style"><Data ss:Type="String">unique</Data></Cell>
<Cell ss:StyleID="header_style"><Data ss:Type="String">not null</Data></Cell>
<Cell ss:StyleID="header_style"><Data ss:Type="String">never null</Data></Cell>
<Cell ss:StyleID="header_style"><Data ss:Type="String">visible</Data></Cell>
<Cell ss:StyleID="header_style"><Data ss:Type="String">mandatory</Data></Cell>
<Cell ss:StyleID="header_style"><Data ss:Type="String">enterable</Data></Cell>
<Cell ss:StyleID="header_style"><Data ss:Type="String">modifiable</Data></Cell>
</Row>
 <xsl:apply-templates select="//table" />
 </Table>
 </Worksheet>
 </Workbook>
</xsl:template>
<xsl:template match="//editor_field_info">
<xsl:variable name="style">
<xsl:choose>
<xsl:when test="parent::node()/@visible='false'">
<xsl:value-of select="$invisible_field_style_pattern" />
</xsl:when>
<xsl:otherwise>
<xsl:value-of select="'Solid'" />
</xsl:otherwise>
</xsl:choose>
</xsl:variable>
<xsl:for-each select="./color">
<Style ss:ID="{ancestor::*[ name() = 'field']/@uuid}">
<Interior ss:Color="{concat( '#', substring-after( exsl4D:format-number(@red, '&amp;x') , '0x00' ), 
substring-after( exsl4D:format-number(@green, '&amp;x') , '0x00' ), 
substring-after( exsl4D:format-number(@blue, '&amp;x') , '0x00' ) ) }" ss:Pattern="{$style}" />
</Style>
</xsl:for-each> 
</xsl:template>
<xsl:template match="//editor_table_info">
<xsl:variable name="style">
<xsl:choose>
<xsl:when test="parent::node()/@visible='false'">
<xsl:value-of select="$invisible_table_style_pattern" />
</xsl:when>
<xsl:otherwise>
<xsl:value-of select="'Solid'" />
</xsl:otherwise>
</xsl:choose>
</xsl:variable>
<xsl:for-each select="./color">
<xsl:choose>
<xsl:when test="((@red = 0) and (@green = 0)) and (@blue = 0)">
<Style ss:ID="{ancestor::*[ name() = 'table']/@uuid}">
</Style>
</xsl:when>
<xsl:otherwise>
<Style ss:ID="{ancestor::*[ name() = 'table']/@uuid}">
<Interior ss:Color="{concat( '#', substring-after( exsl4D:format-number(@red, '&amp;x') , '0x00' ), 
substring-after( exsl4D:format-number(@green, '&amp;x') , '0x00' ), 
substring-after( exsl4D:format-number(@blue, '&amp;x') , '0x00' ) ) }" ss:Pattern="{$style}" />
</Style>
</xsl:otherwise>
</xsl:choose>
</xsl:for-each> 
</xsl:template>
<xsl:template match="//table">
<xsl:for-each select="./field">
<Row>
<xsl:choose>
<xsl:when test="position() = 1">
<xsl:choose>
<xsl:when test="parent::node()/table_extra/editor_table_info/color">
<Cell ss:StyleID="{parent::node()/@uuid}"><Data ss:Type="String"><xsl:value-of select="parent::node()/@name" /></Data></Cell>
</xsl:when>
<xsl:otherwise>
<Cell><Data ss:Type="String"><xsl:value-of select="parent::node()/@name" /></Data></Cell>
</xsl:otherwise>
</xsl:choose>
<Cell><Data ss:Type="String"><xsl:value-of select="parent::node()/@uuid" /></Data></Cell>
</xsl:when>
<xsl:otherwise>
<xsl:choose>
<xsl:when test="parent::node()/table_extra/editor_table_info/color">
<Cell ss:StyleID="{parent::node()/@uuid}" />
</xsl:when>
<xsl:otherwise>
<Cell />
</xsl:otherwise>
</xsl:choose>
<Cell />
</xsl:otherwise>
</xsl:choose>
<xsl:choose>
<xsl:when test="./field_extra/editor_field_info/color">
<Cell ss:StyleID="{@uuid}"><Data ss:Type="String"><xsl:value-of select="@name" /></Data></Cell>
</xsl:when>
<xsl:otherwise>
<Cell><Data ss:Type="String"><xsl:value-of select="@name" /></Data></Cell>
</xsl:otherwise>
</xsl:choose>
<Cell><Data ss:Type="String"><xsl:value-of select="@uuid" /></Data></Cell>
<Cell><Data ss:Type="String">
<xsl:choose>
<xsl:when test="@type = 1">
<xsl:value-of select="'boolean'" />
</xsl:when>
<xsl:when test="@type = 3">
<xsl:value-of select="'integer'" />
</xsl:when>
<xsl:when test="@type = 4">
<xsl:value-of select="'long integer'" />
</xsl:when>
<xsl:when test="@type = 5">
<xsl:value-of select="'integer 64 bits'" />
</xsl:when>
<xsl:when test="@type = 6">
<xsl:value-of select="'real'" />
</xsl:when>
<xsl:when test="@type = 7">
<xsl:value-of select="'float'" />
</xsl:when>
<xsl:when test="@type = 8">
<xsl:value-of select="'date'" />
</xsl:when>
<xsl:when test="@type = 9">
<xsl:value-of select="'time'" />
</xsl:when>
<xsl:when test="@type = 12">
<xsl:value-of select="'picture'" />
</xsl:when>
<xsl:when test="@type = 14">
<xsl:value-of select="'text'" />
</xsl:when>
<xsl:when test="@type = 15">
<xsl:value-of select="'subtable'" />
</xsl:when>
<xsl:when test="@type = 16">
<xsl:value-of select="'subtable'" />
</xsl:when>
<xsl:when test="@type = 18">
<xsl:value-of select="'blob'" />
</xsl:when>
<xsl:when test="@type = 10">
<xsl:choose>
<xsl:when test="@limiting_length">
<xsl:value-of select="concat('alpha', string(@limiting_length))" />
</xsl:when>
<xsl:otherwise>
<xsl:value-of select="'text'" />
</xsl:otherwise>
</xsl:choose>
</xsl:when>
</xsl:choose>
</Data></Cell>
<Cell><Data ss:Type="String">
<xsl:if test="@unique">
<xsl:value-of select="@unique" />
</xsl:if> 
</Data></Cell>
<Cell><Data ss:Type="String">
<xsl:if test="@not_null">
<xsl:value-of select="@not_null" />
</xsl:if> 
</Data></Cell>
<Cell><Data ss:Type="String">
<xsl:if test="@never_null">
<xsl:value-of select="@never_null" />
</xsl:if> 
</Data></Cell>
<Cell><Data ss:Type="String">
<xsl:if test="field_extra/@visible">
<xsl:value-of select="field_extra/@visible" />
</xsl:if> 
</Data></Cell>
<Cell><Data ss:Type="String">
<xsl:if test="field_extra/@mandatory">
<xsl:value-of select="field_extra/@mandatory" />
</xsl:if> 
</Data></Cell>
<Cell><Data ss:Type="String">
<xsl:if test="field_extra/@enterable">
<xsl:value-of select="field_extra/@enterable" />
</xsl:if> 
</Data></Cell>
<Cell><Data ss:Type="String">
<xsl:if test="field_extra/@modifiable">
<xsl:value-of select="field_extra/@modifiable" />
</xsl:if> 
</Data></Cell>
</Row>
</xsl:for-each>
</xsl:template>
</xsl:stylesheet>
